-- Written by Frank Ward
-- http://www.frankward.net
--
--

require "controls.lua"

--[[ Set the Window Caption and Declare initial globals ]]--

function love.load()

	love.graphics.setCaption("circletri")

	new = 1
	score = 0
	speed = .15
	angle = .05
	shipLoc = { 0, 0 }
	bulletList = { }
	enemyList = { }
	attacktimer = 0
	enemytimer = math.random(100,400)
-- Debug Counter
	t = 1
	


--[[ Define initial ship shape. These are also used for the local rotation functions ]]--

	shipVert1 = { -5, 10 }
	shipVert2 = { 5, 10 }
	shipVert3 = { 0, -10 }
end

--[[ Main Update Function ]]--

function love.update(dt)

--[[ Calculate the new ship heading and location based off the last know pos ]]--
	
	shipRear = { ((shipVert1[1] + shipVert2[1]) / 2) , ((shipVert1[2] + shipVert2[2]) / 2) }
	shipVector = { (shipVert3[1] - shipRear[1]) , (shipVert3[2]-shipRear[2]) }
	

--[[ Catch Keyboard inputs ]]--

	attacktimer = attacktimer - 1

	if love.keyboard.isDown("right") then
		
		shipRotateRight()		

	end

	if love.keyboard.isDown("left") then

		shipRotateLeft()

	end

	if love.keyboard.isDown("up") then
		shipForward()
	end

	if love.keyboard.isDown("down") then
		shipBackward()
	end
	
	if love.keyboard.isDown("q") then
		new = 1
		score = 0
	end

	if love.keyboard.isDown(" ") and attacktimer <=0 then
		attacktimer = 15
		fireBullet()
	end


--[[ Check to make sure new positions are valid and inside the playspace ]]--

	boundsCheckX(shipLoc)
	boundsCheckY(shipLoc)
	bulletBounds(bulletList)
	enemyUpdate()
	bulletUpdate()
	checkCollisionShip(shipLoc)
	checkCollisionBullets()
	

--[[ Check to see if it's a new game. If so, reset the pos to center screen ]]--

	newGame()

--[[ Create some enemies if the timer has run out, or decrement the timer ]]--
	
	if enemytimer<= 0 then
		enemytimer = math.random(10,100)
		createEnemy()
	else
	   enemytimer = enemytimer -1
	end

end


function love.draw()

--[[ Store the updated ship dir + pos from the update loop as ship array ]]--	

	ship = { shipVert1[1]+shipLoc[1], shipVert1[2]+shipLoc[2], shipVert2[1]+shipLoc[1], shipVert2[2]+shipLoc[2], shipVert3[1]+shipLoc[1], shipVert3[2]+shipLoc[2] }

--[[ Update Bullet and Enemy Positions ]]--

	
	

	

--[[ Draw everything ]]--
	
	--Draw the Ship
	love.graphics.polygon("fill", ship)
	enemyDraw()
	bulletDraw()

	
	--Draw the World Bounds
	love.graphics.polygon("line", 20, 20, 20, 580, 780, 580, 780, 20)

	--Draw the score
	love.graphics.print( "score: " .. score, 700, 5)	

	--Draw Instructions
	love.graphics.print( "Arrow Keys - Move  |  Space - Fire  |  'q' - Reset", 20, 5)
	love.graphics.print( "Frank Ward", 710, 585)
	
--[[ Draw debug text ]]--
	love.graphics.print( "loc x: " .. shipLoc[1], 20, 585)
	love.graphics.print( "loc y: " .. shipLoc[2], 250, 585)
	love.graphics.print( "# of bullets: " .. #bulletList, 550, 585)
	love.graphics.print( "# of enemies: " .. #enemyList, 420, 585)

end

--[[ Define Functions ]]--

function boundsCheckX(s)

	if s[1] <= 20 then
		s[1] = 20
	elseif s[1] >= 780 then
		s[1] = 780
	end
end

function boundsCheckY(r)	

	if r[2] <= 20 then
		r[2] = 20
	elseif r[2] >= 580 then
		r[2] = 580
	end
end

function bulletBounds(q)
	
	for i in ipairs(q) do	
		if  bulletList[i][1] <= 20 or bulletList[i][1] >= 780 or
			bulletList[i][2] <= 20 or bulletList[i][2] >= 580 then
			
			table.remove(bulletList, i)
		
		end
	end
	

end


function newGame()

	if new == 1 then
		shipLoc = {400, 300}
		new = 0
		bulletList = { }
		enemyList = { }
		t = 1
	else
		return	
	end
	
end

function bulletUpdate()

	for i in ipairs(bulletList) do	
		if bulletList[i] == nil then
			
		else
	
		bulletList[i][1] = bulletList[i][1] + (bulletList[i][3] * .8)
		bulletList[i][2] = bulletList[i][2] + (bulletList[i][4] * .8)
		love.graphics.circle("fill", bulletList[i][1], bulletList[i][2], 2, 5) 
		
		end
	end

end

function bulletDraw()

	for i in ipairs(bulletList) do	
		if bulletList[i] == nil then
			
		else

		love.graphics.circle("fill", bulletList[i][1], bulletList[i][2], 2, 5) 
		end
	end

end

function createEnemy()
	
	local enemyInfo = { math.random( 70, 730), math.random(70, 530), math.random(-5, 5), math.random(-5, 5), math.random(10,50), (math.random(1, 5) * .1), math.random(5,9) }
	
	for i=1, #enemyList + 1 do
		
		if enemyList[i] == nil then
			enemyList[i] = enemyInfo
				
		end
	end
end

function enemyUpdate()

	for i in ipairs(enemyList) do	
		if enemyList[i] == nil then
			
		else
		boundsCheckEnemyX(enemyList[i])
		boundsCheckEnemyY(enemyList[i])
		enemyList[i][1] = enemyList[i][1] + (enemyList[i][3] * enemyList[i][6])
		enemyList[i][2] = enemyList[i][2] + (enemyList[i][4] * enemyList[i][6])
		end
	end

end

function enemyDraw()

	for i in ipairs(enemyList) do	
		if enemyList[i] == nil then
			
		else
		love.graphics.circle("line", enemyList[i][1], enemyList[i][2], enemyList[i][5], enemyList[i][7]) 
		end
	end

end
function boundsCheckEnemyX(s)

	if (s[1] - s[5]) <= 20 then
		s[3] = s[3] * -1
	elseif (s[1] + s[5]) >= 780 then
		s[3] = s[3] * -1
	end
end

function boundsCheckEnemyY(r)	

	if r[2] - r[5] <= 20 then
		r[4] = r[4] * -1
	elseif r[2] + r[5] >= 580 then
		r[4] = r[4] * -1
	end
end

function checkCollisionShip(a)
	for i in ipairs(enemyList) do
		if (a[1] >= enemyList[i][1] - (enemyList[i][5] * .9) and
			a[1] <= enemyList[i][1] + (enemyList[i][5] * .9) and
			a[2] >= enemyList[i][2] - (enemyList[i][5] * .9) and
			a[2] <= enemyList[i][2] + (enemyList[i][5] * .9) )
		then
			table.remove(enemyList, i)
			shipLoc[1] = 400
			shipLoc[2] = 300
			score = score - 100
		end
	end
end

function checkCollisionBullets()
	for i in ipairs(enemyList) do
		for j in ipairs(bulletList) do		
			if (bulletList[j][1] >= enemyList[i][1] - enemyList[i][5] and
				bulletList[j][1] <= enemyList[i][1] + enemyList[i][5] and
				bulletList[j][2] >= enemyList[i][2] - enemyList[i][5] and
				bulletList[j][2] <= enemyList[i][2] + enemyList[i][5] )
			then
				table.remove(enemyList, i)
				table.remove(bulletList, j)
				love.graphics.print( "CollisionBullet", 500, 15)
				love.graphics.print( i .. "  " .. j, 500, 35)
				score = score + 10
				
			end
		end
	end

end
